@@ -60,8 +60,12 @@ class CmbTable {
 	
 	Rcpp::NumericVector updateFromMatrix(const Rcpp::IntegerMatrix& int_cmbs,
 		double incr);
+
+	Rcpp::NumericVector updateFromMatrixByRow(
+		const Rcpp::IntegerMatrix& int_cmbs, double incr);
 	
 	Rcpp::DataFrame asDataFrame() const;
+	Rcpp::NumericMatrix asMatrix() const;
 };
 
 RCPP_EXPOSED_CLASS(CmbTable)
